# $Log: htmlutils.py,v $
# Revision 1.6  2000/05/04 08:43:00  itamar
# *** empty log message ***
#
# Revision 1.5  2000/04/24 11:57:00  itamar
# *** empty log message ***
#
# Revision 1.4  2000/04/24 11:47:56  itamar
# Fixed start_meta in html parser.
#
# Revision 1.3  2000/04/24 10:31:58  itamar
# Finished handling of keywords and description in <meta> tags.
#
# Revision 1.2  2000/04/24 10:10:55  itamar
# Extract keywords and description from <meta>.
#
#

import string, re
from sgmllib import SGMLParser
import config

def join_attrs(attrs):
   attr_list = []
   for attrname, value in attrs:
      attr_list.append('%s="%s"' % (attrname, string.strip(value)))

   if attr_list:
      s = " " + string.join(attr_list, " ")
   else:
      s = ""

   return s


class HeadParser(SGMLParser):

   from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
   
   def __init__(self, removejs=0):
      SGMLParser.__init__(self)
      
      self.removejs = removejs
      
      self.seen_starthead = 0
      self.seen_endhead   = 0
      self.seen_startbody = 0

      self.head = ""
      self.title = ""
      self.bodyattribs = ""
      self.accumulator = ""


   def handle_data(self, data):
      if data:
         self.accumulator = self.accumulator + data

   def handle_comment(self, data):
      if data:
         self.accumulator = self.accumulator + "<!--%s-->" % data


   def handle_charref(self, name):
      self.accumulator = "%s&#%s;" % (self.accumulator, name)

   def handle_entityref(self, name):
      # If it is one of the standard SGML entities,
      # or starts with 'dtml', close it with semicolon
      if self.entitydefs.has_key(name) or string.lower(name[:4]) == 'dtml': 
         x = ';'
      else:
         # this breaks unstandard entities that end with ';'
         x = ''
      self.accumulator = "%s&%s%s" % (self.accumulator, name, x)


   def start_head(self, attrs):
      if not self.seen_starthead:
         self.seen_starthead = 1
         self.head = ""
         self.title = ""
         self.accumulator = ""
         self.keywords = []
         self.description = ""

   def end_head(self):
      if not self.seen_endhead:
         self.seen_endhead = 1
         self.head = string.strip(self.head + self.accumulator)
         self.accumulator = ""
   
   def do_meta(self, attrs):
       d = {}
       for k, v in attrs:
           d[k] = v
       if d.has_key('name') and  d.has_key('content') and d['name'] == 'keywords':
           self.keywords = map(string.strip, 
                               string.split(d['content'], ','))
       if d.has_key('name') and d.has_key('content') and d['name'] == 'description':
           self.description = string.strip(d['content'])
       self.unknown_starttag('meta', attrs)
                                                           
   def start_script(self, attrs):
       if self.removejs and not self.seen_endhead:
           self.head = self.head + self.accumulator
           self.accumulator = ""
       else:
          self.unknown_starttag('script', attrs)

   def end_script(self):
       if (not self.removejs) or self.seen_endhead:
           self.unknown_endtag('script')
       else:
           self.accumulator = ""

   def start_title(self, attrs):
      self.head = self.head + self.accumulator
      self.accumulator = ""

   def end_title(self):
      self.title = string.strip(self.accumulator)
      self.accumulator = ""


   def start_body(self, attrs):
      if not self.seen_startbody:
         self.seen_startbody = 1
         self.accumulator = ""
         self.bodyattribs = join_attrs(attrs)
         # from this point onwards, we no loner need to
         # parse HTML, other than stripping out </body>
         # and </html>.  Especially since this causes
         # problems for <dtml> tags.
         
         # so, stop parsing - only handle_data will be called:
         self.setnomoretags()
         
   # these two shouldn't be called at all:
   def end_body(self): print "end_body" # Do not put </BODY> and </HTML>
   def end_html(self): pass # into output stream


   # Pass other tags unmodified
   def unknown_starttag(self, tag, attrs):
      self.accumulator = self.accumulator + "<%s%s>" % (string.lower(tag), join_attrs(attrs))

   def unknown_endtag(self, tag):
      self.accumulator = self.accumulator + "</%s>" % string.lower(tag)


def clean_html(text):
    # translate Windows-generated html to correct html
    # mainly the ' appearing as ? problem.
    # see http://www.hut.fi/u/jkorpela/www/windows-chars.html
    charmap = string.maketrans("\x91\x92\x93\x94\x96\x97", "''\"\"--") 
    text = string.translate(text, charmap)
    return text

slashbodyre = re.compile(r"</body\s*>", re.I)
slashhtmlre = re.compile(r"</html\s*>", re.I)
   
def parse_html(infile, removejs):
   parser = HeadParser(removejs)

   while 1:
      line = infile.readline()
      if not line: break
      parser.feed(line)

   parser.close()
   infile.close()

   body = parser.accumulator
   
   # get rid of </body> and </html>
   body = slashbodyre.sub("", body)
   body = slashhtmlre.sub("", body)
   
   body = "%s\n%s\n%s\n" % (config.header, body, config.footer)
   
   return parser.title, parser.head, parser.bodyattribs, body, \
          parser.keywords, parser.description
