from wxPython.wx import *
from lsListControl import lsListCtrl
import xmlrpclib, urllib, urlparse

class ZopeListCtrl(lsListCtrl):
    def __init__(self, parent, lID):
        lsListCtrl.__init__(self, parent, lID)
        
        self._objectMapCache = {}
                                        
        tID = NewId()
        self.locationText = wxTextCtrl(self, tID, "", wxDefaultPosition,
                                       wxDefaultSize, wxTE_PROCESS_ENTER)
        tc = wxLayoutConstraints()
        tc.top.SameAs(self, wxTop, 0)
        tc.left.SameAs(self, wxLeft, 0)
        tc.bottom.Absolute(20)
        tc.right.SameAs(self, wxRight, 0)
        self.locationText.SetConstraints(tc)
        EVT_TEXT_ENTER(self, tID, self.LocationTextEnter)
        
        self.InsertColumn(0, "Name")
        self.InsertColumn(1, "Meta Type")
        self.InsertColumn(2, "")
        
        # need to break encapsulation.  oh well
        EVT_RIGHT_DOWN(self._listCtrl, self.onRightDown)
        # for wxMSW
        #EVT_COMMAND_RIGHT_CLICK(self._listCtrl, lID, self.onRightClick)
        # for wxGTK
        EVT_RIGHT_UP(self._listCtrl, self.onRightClick)
        
    def connectZope(self, url, username, password):
        """ Connect to Zope server """
        zopeobj = xmlrpclib.Server(url, username=username, password=password)
        try:
            tmp = zopeobj.objectIds()
        except:
            pass
        else:
            parsedurl = urlparse.urlparse(url)
            self.host = parsedurl[1]
            self.path = parsedurl[2]
            self.username = username
            self.password = password
            self.zopeobj = zopeobj
            self.serverobj = zopeobj
            self.updateList()
    
    def _keycmp(self, key):
        """ factory that returns a function that
        compares two dicts based on the value of
        one of their keys """
        return lambda a, b, key=key: cmp(a[key], b[key])
            
    def updateList(self, nocache = 0):
        lsListCtrl.updateList(self)
        self.locationText.Clear()
        self.locationText.WriteText(self.path or '/')
        wxSetCursor(wxStockCursor(wxCURSOR_WAIT))
        self.DeleteAllItems()
        idcmp = self._keycmp('id')
        if (not self._objectMapCache.has_key(self.host + self.path)) or nocache:
            objects = list(self.zopeobj.objectMap())
            objects.sort(idcmp)
            objects.reverse()
            self._objectMapCache[self.host + self.path] = objects
        else:
            objects = self._objectMapCache[self.host + self.path]
        
        for obj in objects:
            id = obj['id']
            meta = obj['meta_type']
            
            i = self.InsertItem(wxListItem())
            
            if meta in ['Folder', 'ZCatalog']: 
                self.SetStringItem(i, 0, id, 0)
            else:  
                self.SetStringItem(i, 0, id, 1)
            self.SetStringItem(i, 1, meta)
            
            self.SetItemText(i, id)
        
        i = self.InsertItem(wxListItem())
        self.SetStringItem(i, 0, '..')
        for j in range(i):
            self.EnsureVisible(j)
        wxSetCursor(wxStockCursor(wxCURSOR_ARROW))    
    
    def changedir(self, dirname):
        k = self.host + self.path
        if dirname in map(lambda d: d['id'], self._objectMapCache[k]):
            newobj = getattr(self.zopeobj, dirname)
            # test that this is a folderish object (FIXME)
            try: temp = newobj.objectIds()
            except: return 0
            self.zopeobj = newobj
            self.path = "%s/%s" % (self.path, dirname)
            return 1
        elif dirname == '..':
            jn = string.join
            sp = string.split
            # truncate last item in path
            newpath = jn(sp(self.path, '/')[:-1], '/')
            self.path = newpath
            self.zopeobj = xmlrpclib.Server('http://%s/%s/' % (self.host, self.path),
                                     username=self.username, password=self.password)
            
            return 1
        else:
            return 0
    
    def onRightDown(self, event):
        self.x = event.GetX()
        self.y = event.GetY() + 40
        event.Skip()
        
    def onRightClick(self, event):
        menu = wxMenu()
        tPopupID1 = 0
        tPopupID2 = 1
        tPopupID3 = 2
        tPopupID4 = 3
        menu.Append(tPopupID1, "Refresh")
        menu.AppendSeparator()
        menu.Append(tPopupID2, "Delete Selected")
        menu.Append(tPopupID3, "Rename")
        menu.Append(tPopupID4, "New Folder")
        EVT_MENU(self, tPopupID1, self.menuRefresh)
        EVT_MENU(self, tPopupID2, self.menuDeleteSelected)
        EVT_MENU(self, tPopupID3, self.menuRename)
        EVT_MENU(self, tPopupID4, self.menuNewFolder)
        self.PopupMenu(menu, wxPoint(self.x, self.y))
        menu.Destroy()
    
    def menuRefresh(self, event):
        self.updateList()
        
    def menuDeleteSelected(self, event):
        itemNames = map(self.GetItemText, self.selectedItems)
        self.zopeobj.manage_delObjects(itemNames)
        self.updateList(nocache=1)
        
    def menuRename(self, event):
        oldname = self.GetItemText(self.currentItem)
        d = wxDialog(self, -1, "Rename...", size=(300, 100))
        wxStaticText(d, -1, "Rename '%s' to:" % oldname, (10, 10))
        d.newname=wxTextCtrl(d, -1, oldname, (10, 35), (200, 20))
        wxButton(d, wxID_OK, " Rename ", (50, 65)).SetDefault()
        wxButton(d, wxID_CANCEL, " Cancel ", (175, 65))
        val = d.ShowModal()
        if val == wxID_OK:
            newname = d.newname.GetValue()
            self.zopeobj.manage_renameObject(oldname, newname)
            self.updateList(nocache=1)
            
    def menuNewFolder(self, event):
        d = wxDialog(self, -1, "Create new Folder", size=(300, 100))
        wxStaticText(d, -1, "Id", (10, 10))
        d.id=wxTextCtrl(d, -1, '', (70, 10), (200, 20))
        wxStaticText(d, -1, "Title", (10, 35))
        d.title=wxTextCtrl(d, -1, '', (70, 35), (200, 20))
        wxButton(d, wxID_OK, " Create ", (50, 65)).SetDefault()
        wxButton(d, wxID_CANCEL, " Cancel ", (175, 65))
        val = d.ShowModal()
        if val == wxID_OK:
            id = d.id.GetValue()
            title = d.title.GetValue()
            self.zopeobj.manage_addFolder(id, title, 0, 0)
            self.updateList(nocache=1)
        
    def LocationTextEnter(self, event):
        if not hasattr(self, 'zopeobj'): return None
        newpath = self.locationText.GetLineText(0)
        if newpath[-1] == '/': newpath = newpath[:-1]        
        url = "http://%s%s" % (self.host, newpath)
        self.connectZope(url, self.username, self.password)
